import org.snmp4j.*;
import org.snmp4j.mp.MPv3;
import org.snmp4j.security.*;
import org.snmp4j.smi.*;
import org.snmp4j.transport.DefaultUdpTransportMapping;

import java.io.IOException;

/**
 * Created by avaleanu on 30.01.2017.
 */
public class SNMPManager {


    private TransportMapping transport;
    private Snmp snmp;
    private CommandResponder trapHandler;
    private SNMPManagerTrapListener listener;

    public interface SNMPManagerTrapListener{
        void trapReceived(PDU command);
    }

    public SNMPManager(String UDPaddress) throws IOException, InterruptedException {
        transport = new DefaultUdpTransportMapping(new UdpAddress(UDPaddress));
        snmp = new Snmp(transport);
        USM usm = new USM(SecurityProtocols.getInstance(), new OctetString(MPv3.createLocalEngineID()),0);
        SecurityModels.getInstance().addSecurityModel(usm);

        trapHandler = new CommandResponder() {
            @Override
            public void processPdu(CommandResponderEvent event) {
                PDU command = event.getPDU();
                if(command != null){
                    System.out.println(command.toString());
                    listener.trapReceived(command);
                }
            }
        };
    }

    public void startSnmpManager() throws IOException {
        if (transport!=null && snmp !=null){
            transport.listen();
            snmp.addCommandResponder(trapHandler);
            snmp.listen();
        }
    }


    public void setListener(SNMPManagerTrapListener listener) {
        this.listener = listener;
    }
}
