import com.thingworx.communications.client.ConnectedThingClient;
import com.thingworx.communications.client.things.VirtualThing;
import com.thingworx.metadata.annotations.ThingworxEventDefinition;
import com.thingworx.metadata.annotations.ThingworxEventDefinitions;
import com.thingworx.metadata.annotations.ThingworxPropertyDefinition;
import com.thingworx.metadata.annotations.ThingworxPropertyDefinitions;

import com.thingworx.types.collections.ValueCollection;
import com.thingworx.types.primitives.DatetimePrimitive;
import com.thingworx.types.primitives.LocationPrimitive;
import com.thingworx.types.primitives.NumberPrimitive;
import com.thingworx.types.primitives.StringPrimitive;
import org.joda.time.DateTime;
import org.snmp4j.PDU;

import java.io.IOException;

/**
 * Created by avaleanu on 30.01.2017.
 */

@ThingworxPropertyDefinitions(properties = {
    @ThingworxPropertyDefinition(name = "Trap", description = "Basic trap message", aspects = {"isReadOnly:false;pushType:VALUE"})
})
@ThingworxEventDefinitions(events = {
        @ThingworxEventDefinition(name="TrapTriggered", description="A trap has been triggered", dataShape="TrapDataShape", isInvocable=true, isPropertyEvent=false)
})
public class SNMPManagerThing extends VirtualThing implements SNMPManager.SNMPManagerTrapListener {


    private final static String MESSAGE = "message";
    private final static String OID = "oid";
    private final static String GENERIC_TRAP = "genericTrap";
    private final static String TIMESTAMP = "timestamp";

    private String thingName = null;
    private SNMPManager snmpManager = null;


    public SNMPManagerThing(String name, String description, String identifier, ConnectedThingClient client) throws IOException, InterruptedException {
        super(name, description, identifier, client);

        this.thingName = name;
        super.initializeFromAnnotations();

        snmpManager = new SNMPManager("127.0.0.1/162");
        snmpManager.setListener(this);
        snmpManager.startSnmpManager();

    }


    public SNMPManagerThing(String name, String description, ConnectedThingClient client) throws IOException, InterruptedException {
        super(name, description, client);

        this.thingName = name;
        super.initializeFromAnnotations();

        snmpManager = new SNMPManager("127.0.0.1/162");
        snmpManager.setListener(this);
        snmpManager.startSnmpManager();

    }


    // From the VirtualThing class
    // This method will get called when a connect or reconnect happens
    // Need to send the values when this happens
    // This is more important for a solution that does not send its properties on a regular basis
    public void synchronizeState() {
        // Be sure to call the base class
        super.synchronizeState();
        // Send the property values to ThingWorx when a synchronization is required
        super.syncProperties();
    }


    @Override
    public void trapReceived(PDU command) {
        String trapString = command.toString();
        try {
            setProperty("Trap", trapString);
            queueTrapEvent(command);
            this.updateSubscribedProperties(1000);
            this.updateSubscribedEvents(1000);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void queueTrapEvent(PDU command) {
        ValueCollection payload = new ValueCollection();

        // Set values to the fields
        payload.put(MESSAGE, new StringPrimitive(command.getVariableBindings().get(0).toString()));
        payload.put(OID, new StringPrimitive(command.get(0).toString()));
        payload.put(GENERIC_TRAP, new NumberPrimitive(command.getType()));
        payload.put(TIMESTAMP, new DatetimePrimitive(DateTime.now()));

        super.queueEvent("TrapTriggered", new DateTime(), payload);
    }
}
